<?php

class FlexiCache_Wp {

	/**
	* The directory in which this plugin exists, within Wordpress's WP_PLUGIN_DIR
	*/
	const FLEXICACHE_PLUGIN_DIR = 'flexicache';

	/**
	* Add Wordpress actions
	*/
	public static function init ()
	{

		add_action('init', array('FlexiCache','initWpPlugin'));

		if (false == self::isWpAdminPage()) {

			/**
			* Don't continue unless it's an admin page
			*/
			return;

		}

		require_once 'Wp/Admin.php';

		/**
		* Dashboard
		*/
		add_action('right_now_content_table_end', array('FlexiCache_Wp_Admin','renderDashboardRightNowInfo'));

		/**
		* Options page
		*/
		add_action('admin_menu', array('FlexiCache_Wp_Admin','addOptionsPage'));

		/**
		* Hook to display edit box
		*/
		add_action('submitpost_box', array('FlexiCache_Wp_Admin','addPostEditBox'));
		add_action('submitpage_box', array('FlexiCache_Wp_Admin','addPageEditBox'));

		/**
		* Always purge on trash or delete
		*/
		add_action('trash_post', array(__CLASS__, 'purgePost'));
		add_action('delete_post', array(__CLASS__, 'purgePost'));

		if (true == FlexiCache_Config::get('Main','PurgeOnPostPublish')) {

			/**
			* Purge all items on post publish
			*/
			add_action('publish_post', array('FlexiCache','purge'));

		} else {

			/**
			* Just purge the post which was published
			*/
			add_action('publish_post', array(__CLASS__,'purgePost'));

		}

		if (true == FlexiCache_Config::get('Main','PurgeOnPagePublish')) {

			/**
			* Purge all items on page publish
			*/
			add_action('publish_page', array('FlexiCache','purge'));

		} else {

			/**
			* Just purge the page which was published
			*/
			add_action('publish_page', array(__CLASS__,'purgePost'));

		}

	}

	/**
	* Return boolean to indicate whether the current install is Wordpress MU
	* Technique based on http://development.pressible.org/eric/detecting-wpmu
	*/
	public static function isMu ()
	{

		$sMuPluginDir = WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'mu-plugins';

		if (false == file_exists($sMuPluginDir)) {
			return false;
		}

		if (false == is_dir($sMuPluginDir)) {
			return false;
		}

		return true;

	}

	/**
	* Read the plugin version from the plugin file and return a string
	*/
	public static function getPluginVersion ()
	{

		$sPluginFilePath = sprintf("%s/wp-plugin.php", FlexiCache::getPluginDir());

		if (false == file_exists($sPluginFilePath)) {
			return "Can't find plugin file";
		}

		if (false == ($sFile = file_get_contents($sPluginFilePath))) {
			return "Can't read plugin file";
		}

		if (false == preg_match('#Version:\s*([^\r\n]+)[\r\n]#', $sFile, $aCapture)) {
			return "Can't find version information in plugin file";
		}

		return trim($aCapture[1]);

	}

	/**
	* Return boolean to indicate whether caching should be disabled
	* for the current request
	*/
	public static function getDisableCaching ()
	{

		/**
		* Security restriction: If a user is logged in to Wordpress, don't use caching
		*/
		if (true == self::_getIsLoggedIn()) {
			return true;
		}

		/**
		* Security restriction: If the current page is an admin page, return
		* This should be covered by the above rule, but sometimes that one is
		* disabled while testing.
		*/
		if (true == self::isWpAdminPage()) {
			return true;
		}

		/**
		* Don't cache the login page or /wp-admin/ which will redirect to the
		* login page.
		*/
		if (true == preg_match('#^/wp-(admin|login)#', FlexiCache::getRequest()->getUri())) {
			return true;
		}

		foreach ($_COOKIE as $sKey=>$sVal) {

			/**
			* If a comment_author cookie set, don't store the page as it may contain pre-filled
			* comment forms with their details in.
			*/
			if (0 === strpos($sKey, 'comment_author')) {
				return true;
			}

			/**
			* If a passworded page access cookie is set, don't store the page
			*/
			if (0 === strpos($sKey, 'wp-postpass')) {
				return true;
			}

		}

		return false;

	}

	/**
	* Return boolean to indicate whether a user is logged in to Wordpress
	*/
	private static function _getIsLoggedIn ()
	{

		if ((true == function_exists('is_user_logged_in')) && (true == is_user_logged_in())) {
			return true;
		}

		return false;

	}

	/**
	* Return boolean to indicate whether the current page request is
	* for a Wordpress admin page
	*/
	public static function isWpAdminPage ()
	{

		if ((true == function_exists('is_admin')) && (true == is_admin())) {
			return true;
		}

		return false;

	}

	/**
	* Return a post/page path given its id or false if not found
	*/
	public static function getPathFromPostId ($iPostId)
	{

		if (false == $sUrl = get_permalink($iPostId)) {
			return false;
		}

		if (false == ($aUrl = parse_url($sUrl))) {
			return false;
		}

		if (false == (is_array($aUrl))) {
			return false;
		}

		/**
		* parse_url() returns an empty path for "/" so we need to handle
		* this one differently.  First check the response doesn't look
		* 'broken', and if not, assume that empty means "/"
		*/

		if (false == (isset($aUrl['scheme']))) {
			return false;
		}

		if (true == empty($aUrl['path'])) {
			return '/';
		}

		return $aUrl['path'];

	}

	/**
	* Purge the given post from the cache
	*/
	public static function purgePost ($iPostId)
	{

		if (false == ($sPath = self::getPathFromPostId($iPostId))) {
			return false;
		}

		FlexiCache::purgeUri($sPath);

	}

	/**
	* Things to do when the plugin is deactivated
	*/
	public static function deactivatePlugin ()
	{

		/**
		* Remove the standalone .htaccess modification if there is one.
		*/
		/**
		* This is a bad idea for Wordpress MU
		*/
//		require_once 'Wp/Admin/Htaccess.php';
//		FlexiCache_Wp_Admin_Htaccess::remove();

		/**
		* Disable caching in the config
		*/
		$oConfig = FlexiCache_Config::get('Main');
		$oConfig->Enabled = false;

		FlexiCache_Config::save();

	}

}
